# (C) Marc Prager, 2013
# Project: LPC800, THUMB-2 instruction set
#
# tool configuration
#
# Variables used from the environment (if set):
# =============================================
# ARM_EXTRA_CPPFLAGS
#   -DUSE_MX16DIV10
#   -DUSE_MX32DIV10
#
# ARM_TOOL_TRIPLE: defaults to arm-none-eabi
# ARM_OPTIONS: defaults to optimization and maximum warnings/errors
# LDSCRIPT: defaults to 4k RAM controller LPC812
#

PROJECT_MAKEFILE:=$(word $(words ${MAKEFILE_LIST}), ${MAKEFILE_LIST})
PROJECT_DIR:=$(subst /SUFFIX,,$(dir ${PROJECT_MAKEFILE})SUFFIX)

SLICE:=${PROJECT_DIR}/util/slice.pl

ifndef ARM_OPTIONS
ARM_OPTIONS:=-O2 -Wstrict-aliasing=3 -Werror
endif

# specify architecture in preference to cpu
# ARM_ARCH=-march=armv6-m
ARM_ARCH=-mcpu=cortex-m0plus

# should we use ARM or THUMB1/2 instruction set?
ARM_ISET=-mthumb

# allow for multiple tool versions
ifndef ARM_TOOL_TRIPLE
	ARM_TOOL_TRIPLE:=arm-none-eabi
endif

# preprocessor
CPPFLAGS+=${LIB_CPPFLAGS} ${ARM_EXTRA_CPPFLAGS}
#CPPFLAGS+=${LIB_CPPFLAGS} -DUSE_MX16DIV10 -DUSE_MX32DIV10

# assembler
AS:=${ARM_TOOL_TRIPLE}-as
ASFLAGS:=${ARM_EABI} ${LIB_CPPFLAGS}

# C compiler, -Wstrict-aliasing=2 is quite strict
CC:=${ARM_TOOL_TRIPLE}-gcc
CFLAGS:=-ffreestanding ${ARM_ISET} ${ARM_ARCH} -Wall -std=gnu99 -Wno-parentheses \
  ${ARM_OPTIONS} -g
#  -mabi=aapcs -fomit-frame-pointer ${ARM_OPTIONS} -g

# linker
# linking is done by gcc now - in contrast to ld in earlier versions of this Makefile
ifndef LDSCRIPT
	LDSCRIPT=lpc8-4k-ram-c.x
endif

#LD:=${ARM_TOOL_TRIPLE}-gcc -nostartfiles -nodefaultlibs -static ${ARM_ARCH} -Wl,--cref,--trace,--print-map,-Map=mapfile
LD:=${ARM_TOOL_TRIPLE}-gcc -nostartfiles -nodefaultlibs -static ${ARM_ARCH}

%.elf: %.o
	${LD} ${LIB_LDFLAGS} ${LDFLAGS} -T${LDSCRIPT} -o $@ $^ ${LIB_LOADLIBES} ${LOADLIBES} ${LIB_LD_LIBS} ${LDLIBS} -laeabi-cortexm0
#	${LD} ${LIB_LDFLAGS} ${LDFLAGS} -T${LDSCRIPT} -o $@ $^ ${LIB_LOADLIBES} ${LOADLIBES} ${LIB_LD_LIBS} ${LDLIBS} -lgcc

# file format converter
OC:=${ARM_TOOL_TRIPLE}-objcopy
OCFLAGS=--only-section=.text --only-section=.rodata --only-section=.data

# disassembler
OD:=${ARM_TOOL_TRIPLE}-objdump

# symbol lister
NM:=${ARM_TOOL_TRIPLE}-nm

# archiver
AR:=${ARM_TOOL_TRIPLE}-ar

# source file slicer (Perl script)
SLICE:=${PROJECT_DIR}/util/slice.pl

# *.h *.s headers from *.def
DEF2HDR:=${PROJECT_DIR}/util/def2hdr.pl

# generate binary image
%.bin: %.elf
	${OC} ${OCFLAGS} -O binary $< $@

%.hex: %.elf
	${OC} ${OCFLAGS} -O ihex $< $@	

%.rom: %.elf
	${OC} -g -O binary --remove-section=.bss $< $@

%.h: %.def
	${DEF2HDR} $<

%.s: %.def
	${DEF2HDR} $<

