/*
  lpc8/acmp.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

#ifndef lpc8__acmp_h
#define lpc8__acmp_h

/** @file
 * @brief Analog comparator registers.
 */
#include <integers.h>
#include <unionized.h>

/** Analog comparator register interface.
 */
struct PACKED4 Acmp {
	volatile Uint32		ctrl;
	volatile Uint32		lad;
};

extern struct Acmp ACMP;		///< defined in crt0

/** Bits of the ACMP CTRL register.
 */
enum AcmpCtrlBits {
	_ACMP_CTRL_EDGESEL	=3,	///< 2 bits, compedge control
		ACMP_CTRL_EDGESEL_FALLING_	=0,
		ACMP_CTRL_EDGESEL_RISING_	=1,
		ACMP_CTRL_EDGESEL_BOTH_		=2,
		ACMP_CTRL_EDGESEL_BOTH2_	=3,
	_ACMP_CTRL_COMPSA	=6,	///< 1 bits, 0=direct output, 1=output synchronized to AHB bus.
	_ACMP_CTRL_VP_SEL	=8,	///< 3 bits, positive voltage select
		ACMP_CTRL_SEL_LAD_		=0,
		ACMP_CTRL_SEL_ACMP_I1_		=1,
		ACMP_CTRL_SEL_ACMP_I2_		=2,
		ACMP_CTRL_SEL_GND_		=3,	///< not documented, but seen in a schematic of UM.
		ACMP_CTRL_SEL_BANDGAP_		=6,
		// all other reserved
	_ACMP_CTRL_VN_SEL	=11,	///< 3 bits, negative voltage select
		// same bits masks as for VP
	_ACMP_CTRL_EDGECLR	=20,	///< Interrupt clear bit: write 1, then 0.
	_ACMP_CTRL_COMPSTAT	=21,	///< current comparator output level
	_ACMP_CTRL_COMPEDGE	=22,	///< current comparator edge-detect status
	_ACMP_CTRL_HYS		=25,	///< 2 bits, 0=0mV, 1=5mV, 2=10mV, 3=20mV
};

/** Bits of the ACMP LAD register.
 */
enum AcmpLadBits {
	_ACMP_LAD_EN		=0,	///< voltage ladder enable
	_ACMP_LAD_SEL		=1,	///< 5 bits, 0=0*VREF/31, 31=31*VREF/31
	_ACMP_LAD_REF		=6,	///< 1 bit, 0=VREF=VDD, 1=VREF=VDDCMP pin
};


#endif

