/*
  lpc8/mrt.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

#ifndef lpc8__mrt_h
#define lpc8__mrt_h

#include <integers.h>
#include <unionized.h>

union PACKED4 Mrt {
	// as an array of channels
	struct PACKED {
		volatile Uint32		intval;
		const volatile Uint32	timer;
		volatile Uint32		ctrl;
		volatile Uint32		stat;
	} channel[4];

	//.0x00
	unionized struct PACKED { char _0[0x00]; // padding
	volatile Uint32		intval0;
	unionized };
	//.0x04
	unionized struct PACKED { char _1[0x04]; // padding
	const volatile Uint32	timer0;
	unionized };
	//.0x08
	unionized struct PACKED { char _2[0x08]; // padding
	volatile Uint32		ctrl0;
	unionized };
	//.0x0C
	unionized struct PACKED { char _3[0x0C]; // padding
	volatile Uint32		stat0;
	unionized };

	//.0x10
	unionized struct PACKED { char _4[0x10]; // padding
	volatile Uint32		intval1;
	unionized };
	//.0x14
	unionized struct PACKED { char _5[0x14]; // padding
	const volatile Uint32	timer1;
	unionized };
	//.0x18
	unionized struct PACKED { char _6[0x18]; // padding
	volatile Uint32		ctrl1;
	unionized };
	//.0x1C
	unionized struct PACKED { char _7[0x1C]; // padding
	volatile Uint32		stat1;
	unionized };

	//.0x20
	unionized struct PACKED { char _8[0x20]; // padding
	volatile Uint32		intval2;
	unionized };
	//.0x24
	unionized struct PACKED { char _9[0x24]; // padding
	const volatile Uint32	timer2;
	unionized };
	//.0x28
	unionized struct PACKED { char _10[0x28]; // padding
	volatile Uint32		ctrl2;
	unionized };
	//.0x2C
	unionized struct PACKED { char _11[0x2C]; // padding
	volatile Uint32		stat2;
	unionized };

	//.0x30
	unionized struct PACKED { char _12[0x30]; // padding
	volatile Uint32		intval3;
	unionized };
	//.0x34
	unionized struct PACKED { char _13[0x34]; // padding
	const volatile Uint32	timer3;
	unionized };
	//.0x38
	unionized struct PACKED { char _14[0x38]; // padding
	volatile Uint32		ctrl3;
	unionized };
	//.0x3C
	unionized struct PACKED { char _15[0x3C]; // padding
	volatile Uint32		stat3;
	unionized };

	//.0xF4
	unionized struct PACKED { char _16[0xF4]; // padding
	const volatile Uint32	idleCh;		///< first idle channel offset
	unionized };
	//.0xF8
	unionized struct PACKED { char _17[0xF8]; // padding
	volatile Uint32		irqFlag;
	unionized };
};

extern union Mrt MRT;	///< defined in crt0

enum MrtIntvalBits {
	_MRT_INTVAL_LOAD	=31,	///< causes immediate reload/restart.
};

enum MrtCtrlBits {
	_MRT_CTRL_INTEN		=0,	///< set to one for TimerN interrupt
	_MRT_CTRL_MODE		=1,	///< 2 bits: 0=repeat interrupt, 1=one-shot, other: reserved.
};

enum MrtStatusBits {
	_MRT_STATUS_INTFLAG	=0,	///< indicates pending interrupt
	_MRT_STATUS_RUN		=1,	///< 0=timer stopped, 1=timer running
};

enum MrtIdleChBits {
	_MRT_IDLE_CH_CHAN	=4,	///< 3bits of lowest number idle channel; value=4 if all busy
};

#endif

