/*
  lpc8/nvic.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

#ifndef lpc8__nvic_h
#define lpc8__nvic_h

/**
 * @file
 * @brief NVIC and interrupt handler prototypes.
 *
 */

#include <integers.h>
#include <unionized.h>

/** NVIC as a structure at 0xE000E000 (SCS system control space).
 */
extern union PACKED4 {
	//.0x100
	unionized struct PACKED { char _0[0x100]; // padding
	volatile Uint32	setEna;		///< interrupt set enable
	unionized };
	//.0x180
	unionized struct PACKED { char _1[0x180]; // padding
	volatile Uint32	clrEna;		///< interrupt clear enable
	unionized };
	//.0x200
	unionized struct PACKED { char _2[0x200]; // padding
	volatile Uint32	setPend;	///< interrupt set pending
	unionized };
	//.0x280
	unionized struct PACKED { char _3[0x280]; // padding
	volatile Uint32	clrPend;	///< interrupt clear pending
	unionized };
	//.0x400
	unionized struct PACKED { char _4[0x400]; // padding
	volatile Uint32 ipr[8];		///< interrupt priority levels
	unionized };
} NVIC;

enum Lpc8InterruptSources {
	_NVIC_SPI0		=0,
	_NVIC_SPI1		=1,
	_NVIC_USART0		=3,
	_NVIC_USART1		=4,
	_NVIC_USART2		=5,
	_NVIC_I2C0		=8,
	_NVIC_SCT		=9,
	_NVIC_MRT		=10,
	_NVIC_CMP		=11,
	_NVIC_WDT		=12,
	_NVIC_BOD		=13,
	_NVIC_WKT		=15,

	_NVIC_PININT0		=24,
	_NVIC_PININT1,
	_NVIC_PININT2,
	_NVIC_PININT3,
	_NVIC_PININT4,
	_NVIC_PININT5,
	_NVIC_PININT6,
	_NVIC_PININT7,
};

void handlerSpi0(void);
void handlerSpi1(void);
void handlerU0(void);

void handlerU1(void);
void handlerU2(void);

void handlerI2c0(void);
void handlerSct(void);
void handlerMrt(void);
void handlerCmp(void);

void handlerWdt(void);
void handlerBod(void);
void handlerWkt(void);

void handlerPinInt0(void);
void handlerPinInt1(void);
void handlerPinInt2(void);
void handlerPinInt3(void);

void handlerPinInt4(void);
void handlerPinInt5(void);
void handlerPinInt6(void);
void handlerPinInt7(void);

#endif

