/*
  lpc8/romi2c.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

#ifndef lpc8__romi2c_h
#define lpc8__romi2c_h

#include <integers.h>
#include <unionized.h>

#include <lpc8/i2c.h>		// I2C register interface

typedef void *Romi2cHandle;		///< handle type

typedef enum {
	ROMI2C_ERROR_SUCCESS			=0,
	ROMI2C_ERROR_GENERAL			=1,
	ROMI2C_ERROR_NAK			=0x00060001,
	ROMI2C_ERROR_BUFFER_OVERFLOW		=0x00060002,
	ROMI2C_ERROR_BYTE_COUNT			=0x00060003,
	ROMI2C_ERROR_LOSS_OF_ARBRITATION	=0x00060004,
	ROMI2C_ERROR_SLAVE_NOT_ADDRESSED	=0x00060005,
	ROMI2C_ERROR_LOSS_OF_ARB_NAK		=0x00060006,
	ROMI2C_ERROR_BUS			=0x00060007,	///< failure on I2C bus
	ROMI2C_ERROR_CLOCK			=0x00060008,	///< invalid clock frequency
} Romi2cError;

typedef enum {
	ROMI2C_STATE_IDLE		=0,
	ROMI2C_STATE_MASTER_WRITE	=1,
	ROMI2C_STATE_MASTER_READ	=2,
	ROMI2C_STATE_SLAVE_WRITE	=3,
	ROMI2C_STATE_SLAVE_READ		=4,
} Romi2cState;

typedef void Romi2cCallback(Uint32 error, Uint32 n);

typedef struct PACKED4 {
	Uint32			nWrite;
	Uint32			nRead;
	Uint8*			bufferWrite;
	Uint8*			bufferRead;
	Romi2cCallback*		callback;
	Uint8			stopFlag;
	Uint8			dummy[3];	///< padding
} Romi2cParam;

typedef struct PACKED4 {
	Uint32	nWrite;
	Uint32	nRead;
} Romi2cResult;

typedef Romi2cError Romi2cIoFunction(Romi2cHandle, Romi2cParam*, Romi2cResult*);

typedef struct {
	void			(*isrHandler)(Romi2cHandle, Romi2cParam*, Romi2cResult*);
	Romi2cIoFunction*	masterWriteBlocking;
	Romi2cIoFunction*	masterReadBlocking;
	Romi2cIoFunction*	masterWriteReadBlocking;
	Romi2cIoFunction*	masterWrite;
	Romi2cIoFunction*	masterRead;
	Romi2cIoFunction*	masterWriteRead;
	Romi2cIoFunction*	slaveReadBlocking;
	Romi2cIoFunction*	slaveWriteBlocking;
	Romi2cIoFunction*	slaveRead;
	Romi2cIoFunction*	slaveWrite;
	Romi2cError		(*setSlaveAddress)(Romi2cHandle, Uint32 addr0to4, Uint32 mask0to4);
	Uint32			(*getMemSize)(void);
	Romi2cHandle		(*init)(I2c*, Uint32* ram);
	Romi2cError		(*setClock)(Romi2cHandle handle, Uint32 pclk, Uint32 rate);
	Uint32			(*getFirmwareVersion)(void);
	Romi2cState		(*getState)(Romi2cHandle handle);
	Romi2cError		(*setTimeout)(Romi2cHandle handle, Uint32 timeout16I2cClks);
} Romi2c;

#endif

